/*Основа: v1.3.4_@Dmitrysh _____10 профилей по 4 шага в каждом.
  v1.3.3 в редакции @SOLOway & @Watashi от 04-07-2022.  
  Тестирование проводится на платах arduino NANO и ProMINI, на
  которых доступен pin A6 !!! Не забудьте выбрать в начале скетча строку с 
  адресом именно вашего дисплея LCD 2004 i2c !!!
  Все используемые библиотеки имеются в архиве с данным релизом скетча. Успешная
  компиляция проверена в версии arduino IDE 1.8.15. Во избежание недоразумений,
  удалите установленные у вас версии таких библиотек, и используйте вложенные
  в архив, и указанную версию arduino IDE.

  Профили сохраняются в виде структуры, по предложению @Watashi.
  Параметры ПИД для НИ и ВИ задаются в меню отдельно для каждого профиля, 
  что позволяет оперативно менять верхний излучатель на тот, 
  который более удобен для конкретной работы.
  
  @Watashi существенно изменена и оптимизирована работа с кнопками.
  Для работы с кнопками теперь используются внешние библиотеки, в том числе и
  библиотека обработчика резистивной клавиатуры, написанная @Watashi.
  Кнопки получили интуитивно понятные названия: ВВЕРХ/ВНИЗ/ВПРАВО/ВЛЕВО/ОК.
  Долгое нажатие ОК в IDLE - вход в меню настроек.
  Долгое нажатие ОК в меню настроек - сохранение профиля, при этом на дисплей
  выводится соответствующая надпись и звучит короткий сигнал.
  Перемещение по пунктам меню - кнопки ВПРАВО (RIGHT) и ВЛЕВО (LEFT).
  Изменение значения параметра меню - кнопки ВВЕРХ (UP) и ВНИЗ (DOWN).
  В режиме IDLE можно включать охладители платы нажатием кнопки ВПРАВО (RIGHT), и
  отключать их кнопкой ВЛЕВО (LEFT). Охладители платы НЕ стартуют автоматически
  по окончании профиля.
  В режиме IDLE, при нажатии ВЛЕВО (LEFT) 5+ секунд, есть инициализация
  "тестового примерного" профиля, при этом текущий профиль перезапишется
  значениями из "тестового примерного".
  Во избежание отображения некорректных значений параметров на LCD, рекомендуется 
  каждый из 10 профилей заменить значениями предустановленного встроенного профиля. 
  Предустановленные параметры всего лишь примерные, взяты со станции, которой давно 
  нет, поэтому необходимо индивидуально их настроить на вашей конкретной станции.
  
  Реализовано задание в меню следующих параметров:
  а) скорость роста температуры (рампа) для НИ (0,1-3,0°С/сек)
   (респект @Dmitrysh!) и для ВИ  (0,1-3,0°С/сек); 
  б) время удержания уставки НИ "полочка" 
   (0-255сек) перед моментом старта ВИ; 
  в) уставка НИ (80-330°C) с шагом 10°С;
  г) уставки шагов ВИ до 255°С с шагом 1°С;
  д) минимальное и максимальное значение мощности ВИ в "%"
   (0-99) для каждого шага ВИ;
  е) минимальное и максимальное значение мощности НИ в "%"
   (0-99) для каждого профиля;
  ж) число шагов ВИ в профиле (1-4);
  з) мощность на этапе преднагрева НИ в "%" (0-50)(смотри примечание);
  и) уставка преднагрева платы (с помощью НИ, после старта пайки) 
   по датчику НИ сверху на краю платы (0-100°С) (смотри примечание);
  к) значения коэффициентов ПИД НИ и ВИ (0-255) с шагом 1;
  л) время удержания уставки каждого шага ВИ "полочка" (0-255сек)

  Примечание: Пункты з) и и) контроллер отработает только если ДО компиляции, во
  вкладке Setting вы раскомментировали строку [ #define inert_heater 1 ].
  
  Некоторые настройки скетча, кроме параметров меню, вынесены во вкладку Setting.
  Их можно редактировать, включать и отключать некоторые функции, задавать значения
  константам функций, изменять пины подключения периферии, только перед компиляцией:
   а) Есть выбор варианта ПИД для НИ (для инерционных/для быстрых нагревателей) -
   (по умолчанию активирован ПИД НИ для инерционных керамических излучателей (респект @geleos27!).
   Логика ПИД ВИ (также, как и для "быстрых" НИ на основе линейных галогеновых ламп)
   заменена на предложенную @Dmitrysh (https://clck.ru/e58xt) - "работать не по конечному
   значению ошибки, а по измерению температуры."). Если НИ на линейных галогеновых
   лампах - нужно закомментировать строку #define LongBufferPid 1.
   Внутри функций ПИД присутствуют делители/множители для составляющих P.I.D.. 
   Внимательно изучите их величины!!! Сейчас в функции PID1 для ВИ установлены 
   такие делители/множители:
   P/10 (т.е. в меню 255 будет равно 25,5 и 1 будет равен 0,1);
   I/1000;
   D - без делителя.
   В функции PID2 для НИ, в варианте для "быстрых" излучателей:
   P/10;
   I/1000;
   D - без делителя.
   В функции PID2 с буфером, для инерционных керамических НИ:
   P/10;
   I/10000;
   D  - без делителя.
   б) Есть выбор между обычными одиночными (5 штук) кнопками и
   аналоговой резистивной клавиатурой с 5-ю кнопками, аналоговая - на pin. А6.
   Если кнопки одиночные - нужно закомментировать строку #define resist_keyboard 1
   в) Есть выбор работы с прерыванием от внешней схемы ZCC, или с прерыванием по
   внутреннему таймеру. При работе с прерыванием от схемы контроля перехода напряжения
   сети через ноль вольт (ZCC), параметры для attachInterrupt нужно задать под свою
   версию схемы (FALLING или RISING) в конце блока void setup(). Во вкладке Setting
   нужно указать пин подключения для схемы ZCC - 2, или 3, это строки
   #define SetInterrupt перед секцией ПИД. Для пин 2, или пин 3 в скетче, при
   выборе прерывания от внешней схемы ZCC в скетче включается подтягивающий к +5в
   внутренний резистор - обратите на это внимание!!!
   г) Теперь есть выбор типа для функции ПРЕДНАГРЕВ НИ при старте пайки. Если НИ
   на галогеновых линейных лампах, закомментируйте строку #define inert_heater 1,
   и после компиляции будет активирован преднагрев при старте пайки
   с 3% мощности НИ в течении 5 секунд, для уменьшения пускового тока холодных
   галогеновых ламп. Если оставить эту строку раскомментированной, то после компиляции
   будет активирован преднагрев НИ для инерционных излучателей до задаваемой в меню
   t°C pTarg по датчику НИ на плате, и задаваемой там же фиксированной мощностью pPwr
   По окончании этапа преднагрев НИ для инерционных излучателей прозвучит
   звуковой сигнал и НИ перейдёт под управление своего ПИД.
   д) По умолчанию на монитор компьютера/ноутбука можно выводить графики
   температура/время и мощность нагревателей/время. В процессе настройки станции
   также можно строить графики влияния составляющих PID НИ, или ВИ в программу
   SerialPortPlotter, модифицированную @geleos27:
   https://drive.google.com/drive/folders/1MJODRkzjEhu9_pDdhrgusmGDYtArU51c
   выбор набора выводимых данных производится заменой функций [ BufferUpload(); ],
   [ BufferUpload2(); ], [ BufferUpload3(); ], в блоке кода: //включение нагревателей
   Если нравится программа irsp © от @Dmitrysh, а не SerialPortPlotter - нужно 
   закоментировать строку #define SerialPortPlotter 1.
   е) За сколько градусов до достижения НИ своей уставки должен включиться в работу ВИ,
   можно задать в строке #define delta 10 (по умолчанию значение 10 градусов). (По факту, 
   это не работает из-за того, что появилось другое условие старта ВИ сразу по окончании 
   заданной в меню паузы на уставке НИ. В это время, если датчик ВИ расположен у чипа 
   ближе к центру платы, на нём, вероятнее всего, будет t° несколько выше t° уставки НИ, 
   и алгоритм автоматически пересчитает нужные данные для корректной работы ВИ. Позже поправим.)
   ж) В верхней строке LCD справа можно выводить длительность процесса пайки в секундах,
   раскомментировав строку #define TimePrint 1.

  Если нужно использовать только НИ, например, для просушки платы после отмывки, -
  отведите ВИ в сторону и не устанавливайте датчик ВИ. В этом случае плата будет
  нагрета до уставки НИ для текущего профиля пайки и это состояние будет длиться
  пока вы не установите датчик ВИ, либо не завершите пайку кнопкой ВЛЕВО(LEFT).

  При пропадании контакта любого датчика с модулем MAX6675, в режиме работы по
  профилю, пайка будет автоматически остановлена и запустить её снова возможно лишь
  устранив потерю контакта и отключив, а затем заново включив питание контроллера.
  Важно!!! - Нельзя допускать электрического контакта любого датчика с корпусом станции!!! 
*********
   
   В планах: организация преднагрева для ВИ, изменение отображения меню. */
